/*
 * *##% 
 * EUGene :: EUGene
 * Copyright (C) 2004 - 2009 CodeLutin
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Lesser Public License for more details.
 *
 * You should have received a copy of the GNU General Lesser Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/lgpl-3.0.html>.
 * ##%*
 */
package org.nuiton.eugene.java;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.nuiton.eugene.GeneratorUtil;
import org.nuiton.eugene.models.object.*;
import org.nuiton.eugene.models.object.ObjectModelModifier;
import org.nuiton.eugene.models.object.xml.*;

/**
 * JavaBuilder  TODO heritates it from ModelBuilder
 * <p/>
 * Created: 29 oct. 2009
 *
 * @author fdesbois
 * @version $Revision: 811 $
 *          <p/>
 *          Builder to fill an empty ObjectModel with java specificities (imports, only one inheritance, constructor, ...).
 *          JavaBuilder uses ImportsManagerExtension to manage imports in the model.
 *          JavaBuilder is also based on ObjectModelBuilder for the simple filling of the model.
 *          <p/>
 *          Mise a jour: $Date: 2010-01-29 21:09:16 +0100 (ven., 29 janv. 2010) $
 *          par : $Author: tchemit $
 */
public class JavaBuilder {

    private static final Log log = LogFactory.getLog(JavaBuilder.class);

    /**
     * Builder where the filling is based on
     */
    protected ObjectModelBuilder modelBuilder;

    /**
     * ObjectModel extension to manage imports : one ImportsManager by classifier
     */
    protected ImportsManagerExtension importsManagerExtension;
    /**
     * ObjectModel extension to manage imports : one ImportsManager by classifier
     */
    protected AnnotationsManagerExtension annotationsManagerExtension;
    /**
     * ObjectModel extension to manage constants : one ConstantsManager for all
     */
    protected ConstantsManagerExtension constanssManagerExtension;

    @SuppressWarnings("unchecked")
    public JavaBuilder(String modelName) {
        this.modelBuilder = new ObjectModelBuilder(modelName);

        this.importsManagerExtension = getModel().getExtension(
                ImportsManagerExtension.OBJECTMODEL_EXTENSION,
                ImportsManagerExtension.class);

        this.annotationsManagerExtension = getModel().getExtension(
                AnnotationsManagerExtension.OBJECTMODEL_EXTENSION,
                AnnotationsManagerExtension.class);

        this.constanssManagerExtension = getModel().getExtension(
                ConstantsManagerExtension.OBJECTMODEL_EXTENSION,
                ConstantsManagerExtension.class);
    }

    /**
     * Get the model which is built
     *
     * @return an ObjectModel
     */
    public ObjectModel getModel() {
        return this.modelBuilder.getModel();
    }

    public void setDocumentation(ObjectModelElement element,
                                 String documentation) {
        modelBuilder.setDocumentation(element, documentation);
    }

    /**
     * Add an import to a classifier. Imports are automatic for lots of JavaBuilder's methods except body
     * code of operations.
     * <pre>
     * You can have some complex imports like :
     * - new java.util.List<org.chorem.bonzoms.Role>()
     *          --> two imports 'java.util.List' and 'org.chorem.bonzoms.Role'
     * - java.util.Collection<T extends org.nuiton.topia.TopiaEntity>
     *          --> two imports 'java.util.Collection' and 'org.nuiton.topia.TopiaEntity'
     * </pre>
     *
     * @param classifier where the imports will be added.
     * @param imports    to add
     * @see org.nuiton.eugene.GeneratorUtil#getTypesList(java.lang.String)
     */
    public void addImport(ObjectModelClassifier classifier, String imports) {
        if (imports == null) {
            return;
        }
        ImportsManager manager = importsManagerExtension.getManager(classifier);
        imports = imports.trim();
        // remove ... operator
        if (imports.endsWith("...")) {
            imports = imports.substring(0, imports.length() - 3);
        }
        for (String oneType : GeneratorUtil.getTypesList(imports)) {
            manager.addImport(oneType);
            if (log.isDebugEnabled()) {
                log.debug("Add import for : " + classifier.getQualifiedName() +
                        " _ import: " + oneType);
            }
        }
    }

    /**
     * Add an annotation on an element of a classifier.
     * 
     * @param classifier where the annotation will be added.
     * @param element element on which add annotation
     * @param annotation annotation to add
     */
    public void addAnnotation(ObjectModelClassifier classifier,
                              ObjectModelElement element,
                              String annotation) {
        if (annotation == null) {
            return;
        }
        AnnotationsManager manager =
                annotationsManagerExtension.getManager(classifier);
        annotation = annotation.trim();

        manager.addAnnotation(element, annotation);
        if (log.isDebugEnabled()) {
            log.debug("Add annotation for <" + classifier.getQualifiedName() +
                    ":" + element.getName() + ">  : " + annotation);
        }
    }

    public String getConstantName(String propertyName) {
        String result = constanssManagerExtension.getConstantName(propertyName);
        if (log.isDebugEnabled()) {
            log.debug("get constant name for <" + propertyName + ">  : " +
                    result);
        }
        return result;
    }

    /**
     * Create a new class in the model.
     *
     * @param name
     * @param packageName package's name of the class to create
     * @return a new ObjectModelClass
     * @see org.nuiton.eugene.models.object.ObjectModelBuilder#createClass(java.lang.String, java.lang.String,org.nuiton.eugene.models.object.ObjectModelModifier...)
     */
    public ObjectModelClass createClass(String name, String packageName) {
        return modelBuilder.createClass(name, packageName);
    }

    /**
     * Create a new abstract class in the model.
     *
     * @param name
     * @param packageName package's name of the class to create
     * @return a new ObjectModelClass
     * @see org.nuiton.eugene.models.object.ObjectModelBuilder#createClass(java.lang.String,java.lang.String, org.nuiton.eugene.models.object.ObjectModelModifier...)
     */
    public ObjectModelClass createAbstractClass(String name,
                                                String packageName) {
        return modelBuilder.createClass(name, packageName,
                ObjectModelModifier.ABSTRACT);
    }

    /**
     * Create a new interface in the model
     *
     * @param name
     * @param packageName package's name of the interface to create
     * @return a new ObjectModelInterface
     * @see org.nuiton.eugene.models.object.ObjectModelBuilder#createInterface(java.lang.String, java.lang.String)
     */
    public ObjectModelInterface createInterface(String name,
                                                String packageName) {
        return modelBuilder.createInterface(name, packageName);
    }

    /**
     * Create a new enumration in the model
     *
     * @param name
     * @param packageName package's name of the enumeration to create
     * @return a new ObjectModelEnumeration
     */
    public ObjectModelEnumeration createEnumeration(String name,
                                                    String packageName) {
        return modelBuilder.createEnumeration(name, packageName);
    }

    /**
     * Set the superclass of an other class. Only one superclass can be set to the class.
     * IMPORTS superclassQualifiedName.
     *
     * @param classifier
     * @param superclassQualifiedName
     * @see org.nuiton.eugene.models.object.ObjectModelBuilder#addInterface(
     *org.nuiton.eugene.models.object.ObjectModelClassifier, java.lang.String)
     */
    public void setSuperClass(ObjectModelClass classifier,
                              String superclassQualifiedName) {
        ObjectModelClassImpl impl = (ObjectModelClassImpl) classifier;
        impl.clearSuperclasses(); // suppress all existing superclass: only one can be set for java
        modelBuilder.addSuperclass(impl, superclassQualifiedName);
        this.addImport(classifier, superclassQualifiedName);
    }

    /**
     * Add an interface to a classifier (interface, class, enum).
     * IMPORTS interfaceQualifiedName.
     *
     * @param classifier
     * @param interfaceQualifiedName
     * @see org.nuiton.eugene.models.object.ObjectModelBuilder#addInterface(
     *org.nuiton.eugene.models.object.ObjectModelClassifier, java.lang.String)
     */
    public void addInterface(ObjectModelClassifier classifier,
                             String interfaceQualifiedName) {
        modelBuilder.addInterface(classifier, interfaceQualifiedName);
        this.addImport(classifier, interfaceQualifiedName);
    }

    /**
     * Add a new attribute to a classifier.
     * IMPORTS type, value.
     *
     * @param classifier
     * @param name
     * @param type
     * @param value
     * @param modifiers
     * @return a new ObjectModelAttribute
     * @see org.nuiton.eugene.models.object.ObjectModelBuilder#addAttribute(org.nuiton.eugene.models.object.ObjectModelClassifier, java.lang.String,java.lang.String, java.lang.String, org.nuiton.eugene.models.object.ObjectModelModifier...)
     */
    public ObjectModelAttribute addAttribute(ObjectModelClassifier classifier,
                                             String name,
                                             String type,
                                             String value,
                                             ObjectModelModifier... modifiers) {
        this.addImport(classifier, type);
        this.addImport(classifier, value);
        return modelBuilder.addAttribute(classifier, name, type, value,
                modifiers);
    }

    /**
     * Add a new constant to a classifier. A constant is static and final and have a value.
     * IMPORTS type, value.
     *
     * @param classifier
     * @param name
     * @param type
     * @param value
     * @param visibility modifier allowed : PUBLIC, PRIVATE, PROTECTED, PACKAGE
     * @return a new ObjectModelAttribute
     * @throws IllegalArgumentException if the modifier is not a visibility
     */
    public ObjectModelAttribute addConstant(ObjectModelClassifier classifier,
                                            String name,
                                            String type,
                                            String value,
                                            ObjectModelModifier visibility)
            throws IllegalArgumentException {
        if (!visibility.isVisibility()) {
            throw new IllegalArgumentException("Illegal visibility type : " +
                    visibility.name() +
                    " for " + classifier.getQualifiedName());
        }

        return addAttribute(classifier, name, type, value, visibility,
                ObjectModelModifier.STATIC, ObjectModelModifier.FINAL);
    }

    /**
     * Add a new attribute to a classifier with no default value. Default visibility is set to PROTECTED.
     * IMPORTS type.
     *
     * @param classifier
     * @param name
     * @param type
     * @return a new ObjectModelAttribute
     */
    public ObjectModelAttribute addAttribute(ObjectModelClassifier classifier,
                                             String name,
                                             String type) {
        return addAttribute(classifier, name, type, "",
                ObjectModelModifier.PROTECTED);
    }

    /**
     * Add a new attribute to a classifier from an existing attribute.
     * IMPORTS attribute.getType() and attribute.getDefaultValue().
     *
     * @param classifier
     * @param attribute
     * @return a new ObjectModelAttribute
     */
    public ObjectModelAttribute addAttribute(ObjectModelClassifier classifier,
                                             ObjectModelAttribute attribute) {

        Set<ObjectModelModifier> modifiers = new HashSet<ObjectModelModifier>();

        if (attribute.isStatic()) {
            modifiers.add(ObjectModelModifier.STATIC);
        }
        if (attribute.isFinal()) {
            modifiers.add(ObjectModelModifier.FINAL);
        }
        if (attribute.getVisibility().equals("public")) {
            modifiers.add(ObjectModelModifier.PUBLIC);
        } else if (attribute.getVisibility().equals("protected")) {
            modifiers.add(ObjectModelModifier.PROTECTED);
        } else if (attribute.getVisibility().equals("private")) {
            modifiers.add(ObjectModelModifier.PRIVATE);
        } else if (attribute.getVisibility().equals("")) {
            modifiers.add(ObjectModelModifier.PACKAGE);
        }

        return addAttribute(classifier,
                attribute.getName(),
                attribute.getType(),
                attribute.getDefaultValue(),
                modifiers.toArray(new ObjectModelModifier[modifiers.size()]));
    }

    /**
     * Add a new operation to a classifier.
     *
     * @param classifier
     * @param name
     * @param type
     * @param modifiers
     * @return a new ObjectModelOperation
     * @see org.nuiton.eugene.models.object.ObjectModelBuilder#addOperation(org.nuiton.eugene.models.object.ObjectModelClassifier, java.lang.String,java.lang.String, org.nuiton.eugene.models.object.ObjectModelModifier...)
     */
    public ObjectModelOperation addOperation(ObjectModelClassifier classifier,
                                             String name,
                                             String type,
                                             ObjectModelModifier... modifiers) {
        this.addImport(classifier, type);
        return modelBuilder.addOperation(classifier, name, type, modifiers);
    }

    /**
     * Add a new block to a classifier.
     *
     * @param classifier
     * @param modifiers
     * @return a new ObjectModelOperation
     * @see org.nuiton.eugene.models.object.ObjectModelBuilder#addOperation(org.nuiton.eugene.models.object.ObjectModelClassifier, java.lang.String,java.lang.String, org.nuiton.eugene.models.object.ObjectModelModifier...)
     */
    public ObjectModelOperation addBlock(ObjectModelClassifier classifier, 
                                             ObjectModelModifier... modifiers) {
        ObjectModelOperationImpl operation = (ObjectModelOperationImpl)
                modelBuilder.addOperation(classifier, null, null, modifiers);
        if (Arrays.asList(modifiers).contains(ObjectModelModifier.STATIC)) {
            operation.setStatic(true);
        }
        return operation;
    }

    /**
     * Add a constructor to a class.
     *
     * @param clazz
     * @param visibility
     * @return a new ObjectModelOperation
     * @throws IllegalArgumentException if the modifier is not a visibility
     */
    public ObjectModelOperation addConstructor(ObjectModelClass clazz,
                                               ObjectModelModifier visibility)
            throws IllegalArgumentException {
        if (!visibility.isVisibility()) {
            throw new IllegalArgumentException("Illegal visibility type : " +
                    visibility.name() +
                    " for " + clazz.getQualifiedName());
        }

        return addOperation(clazz, clazz.getName(), null, visibility);
    }

    /**
     * Add a new parameter to an existing operation.
     *
     * @param operation
     * @param type
     * @param name
     * @return a new ObjectModelParameter
     * @see org.nuiton.eugene.models.object.ObjectModelBuilder#addParameter(
     *org.nuiton.eugene.models.object.ObjectModelOperation, java.lang.String, java.lang.String)
     */
    public ObjectModelParameter addParameter(ObjectModelOperation operation,
                                             String type,
                                             String name) {
        this.addImport((ObjectModelClassifier) operation.getDeclaringElement(),
                type);
        return modelBuilder.addParameter(operation, type, name);
    }

    /**
     * Add an exception to an operation.
     *
     * @param operation
     * @param exception
     * @see org.nuiton.eugene.models.object.ObjectModelBuilder#addException(
     *org.nuiton.eugene.models.object.ObjectModelOperation, java.lang.String)
     */
    public void addException(ObjectModelOperation operation, String exception) {
        this.addImport((ObjectModelClassifier) operation.getDeclaringElement(),
                exception);
        modelBuilder.addException(operation, exception);
    }

    /**
     * Set the operation body code.
     *
     * @param operation
     * @param body
     * @throws IllegalArgumentException if operation is abstract
     * @see org.nuiton.eugene.models.object.ObjectModelBuilder#setOperationBody(
     *org.nuiton.eugene.models.object.ObjectModelOperation, java.lang.String)
     */
    public void setOperationBody(ObjectModelOperation operation, String body)
            throws IllegalArgumentException {
        if (operation.isAbstract()) {
            throw new IllegalArgumentException("Unable to add body for an " +
                    "abstract method");
        }
        modelBuilder.setOperationBody(operation, body);
    }

    public ObjectModelClassifier addInnerClassifier(ObjectModelClass clazz,
                                                    ObjectModelType type,
                                                    String name)
            throws IllegalArgumentException {
        return modelBuilder.addInnerClassifier(clazz, type, name);
    }

    public ObjectModelOperation addConstructor(ObjectModelEnumeration enumeration,
                                               ObjectModelModifier visibility)
            throws IllegalArgumentException {
        if (!visibility.isVisibility()) {
            throw new IllegalArgumentException("Illegal visibility type : " +
                    visibility.name() +
                    " for " + enumeration.getQualifiedName());
        }

        return addOperation(enumeration, enumeration.getName(), null, visibility);
    }

    public void addLiteral(ObjectModelEnumeration classifier, String name) {
        modelBuilder.addLiteral(classifier, name);
    }
}
