/*
 * *##% 
 * EUGene :: EUGene
 * Copyright (C) 2004 - 2009 CodeLutin
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Lesser Public License for more details.
 *
 * You should have received a copy of the GNU General Lesser Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/lgpl-3.0.html>.
 * ##%*
 */
package org.nuiton.eugene.writer;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.nuiton.eugene.models.Model;

import java.io.File;
import java.io.IOException;
import java.util.*;

/**
 * Abstract implementation of the {@link ChainedFileWriter}.
 *
 * @author tchemit
 * @since 2.0.0
 */
public abstract class AbstractChainedFileWriter implements ChainedFileWriter {
    /**
     * Logger
     */
    private static final Log log = LogFactory.getLog(AbstractChainedFileWriter.class);
    /**
     * entries to treate with this writer
     */
    protected List<ChainedFileWriterEntry> entries;
    /**
     * previous writer (can be null)
     */
    protected ChainedFileWriter previousWriter;
    /**
     * next writer (can be null)
     */
    protected ChainedFileWriter nextWriter;
    /**
     * universe of authorized properties (keys are property names, values are property descriptions).
     */
    protected final Map<String, String> authorizedPropertyDescriptions;
    /**
     * real properties obtained fro a configuration in {@link #initWriter(ChainedFileWriterConfiguration)} method
     */
    protected Map<String, Object> properties;

    protected AbstractChainedFileWriter(String... propertyNameAndDescriptions) {
        if (propertyNameAndDescriptions.length % 2 != 0) {
            throw new IllegalArgumentException("propertyNameAndDescriptions must be couple of (property key, property description), but was " + Arrays.toString(propertyNameAndDescriptions));
        }
        entries = new ArrayList<ChainedFileWriterEntry>();
        properties = new TreeMap<String, Object>();
        Map<String, String> authorizedPropertyDescriptions = new TreeMap<String, String>();
        for (int i = 0, max = propertyNameAndDescriptions.length / 2; i < max; i++) {
            authorizedPropertyDescriptions.put(propertyNameAndDescriptions[2 * i], propertyNameAndDescriptions[2 * i + 1]);
        }
        this.authorizedPropertyDescriptions = Collections.unmodifiableMap(authorizedPropertyDescriptions);
    }

    /**
     * Generates some files for a given input, output  and include pattern.
     *
     * @param configuration  the shared configuration for all writers
     * @param outputDir      where to generate files
     * @param inputDirectory where to pick up files
     * @param includePattern pattern of files to include
     * @throws IOException for any IO pb.
     */
    protected abstract void generate(ChainedFileWriterConfiguration configuration, File outputDir, File inputDirectory, String includePattern) throws IOException;

    @Override
    public Map<String, String> getAuthorizedPropertyDescriptions() {
        return authorizedPropertyDescriptions;
    }

    @Override
    public String[] getAuthorizedPropertyNames() {
        Set<String> keys = authorizedPropertyDescriptions.keySet();
        return keys.toArray(new String[keys.size()]);
    }

    @Override
    public void clear() {
        entries.clear();
        properties.clear();
        previousWriter = nextWriter = null;
    }

    @Override
    public void addEntry(ChainedFileWriterEntry entry) {
        entries.add(entry);
    }

    @Override
    public <T> T getProperty(String key, Class<T> type) {
        return (T) properties.get(key);
    }

    @Override
    public void generate(ChainedFileWriterConfiguration configuration) throws IOException {
        if (log.isDebugEnabled()) {
            log.debug("[" + getInputProtocol() + "] Start generate with writer " + this);
        }
        initWriter(configuration);

        try {
            File outputDir = getOutputDirectory(configuration.getOutputDirectory(), configuration.isTestPhase());

            if (!outputDir.exists()) {
                if (log.isDebugEnabled()) {
                    log.debug("[" + getInputProtocol() + "] Create output directory " + outputDir);
                }
                boolean b = outputDir.mkdirs();
                if (!b) {
                    throw new IOException("Could not creat directory " + outputDir);
                }
            }

            for (ChainedFileWriterEntry e : entries) {
                if (log.isDebugEnabled()) {
                    log.debug("[" + getInputProtocol() + "] Will generate entry " + e.getInputDirectory() + " : " + e.getIncludePattern());
                }
                generate(configuration, outputDir, e.getInputDirectory(), e.getIncludePattern());
            }
        } finally {
            clear();
        }
    }

    @Override
    public String getInputProtocol(String modelType) {
        // input protocol is the same for all model
        return acceptModel(modelType) ? getInputProtocol() : null;
    }

    @Override
    public File getOutputDirectory(File outputBasedir, boolean testPhase) {
        return new File(outputBasedir, testPhase ? getDefaultTestOutputDirectory() : getDefaultOutputDirectory());
    }

    @Override
    public List<ChainedFileWriterEntry> getEntries() {
        return entries;
    }

    protected ChainedFileWriter getNextWriter() {
        return nextWriter;
    }

    protected ChainedFileWriter getPreviousWriter() {
        return previousWriter;
    }

    protected void setNextWriter(ChainedFileWriter nextWriter) {
        this.nextWriter = nextWriter;
    }

    /**
     * Initialize the writer before the generation.
     *
     * @param configuration the configuration to use for int
     */
    protected void initWriter(ChainedFileWriterConfiguration configuration) {
        Map<String, Object> map = configuration.getProperties();

        for (String key : getAuthorizedPropertyNames()) {
            //TODO-TC-20091217, should prefix keys by the inputProtocol to avoid collisions ?
            if (map.containsKey(key)) {
                // keep this property
                properties.put(key, map.get(key));
            }
        }
    }
}
